import{p as e,f as o}from"./flowDb-956e92f1-596b3df3.js";import{f as t,g as a}from"./styles-c10674c1-b45b8e84.js";import{u as i}from"./index-64550f44.js";import"./graph-ce564af9.js";import"./layout-34133638.js";import"./index-3862675e-65f53377.js";import"./clone-160f349e.js";import"./edges-e0da2a9e-1b1afc6f.js";import"./createText-2e5e7dd3-948eed54.js";import"./line-5cacaef8.js";import"./array-9f3ba611.js";import"./path-53f90ab3.js";import"./channel-746a1de6.js";const M={parser:e,db:o,renderer:t,styles:a,init:r=>{r.flowchart||(r.flowchart={}),r.flowchart.arrowMarkerAbsolute=r.arrowMarkerAbsolute,i({flowchart:{arrowMarkerAbsolute:r.arrowMarkerAbsolute}}),t.setConf(r.flowchart),o.clear(),o.setGen("gen-2")}};export{M as diagram};
