"""Constants for agent tools."""

TOOL_LIST_PROJECT_FILES = "list_project_files"
TOOL_READ_PROJECT_FILES = "read_project_files"
TOOL_GET_PROJECT_FILE = "get_project_file"
TOOL_WRITE_PROJECT_FILE = "write_project_file"
TOOL_UPDATE_MULTIPLE_FILES = "update_multiple_files"

TOOL_CREATE_PLAN = "create_plan"
TOOL_UPDATE_TASK = "update_task"
